package prefuse;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;
import java.awt.geom.Point2D.Double;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Rectangle2D.Double;
import java.awt.image.BufferedImage;
import java.io.OutputStream;
import java.util.Iterator;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.JComponent;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.text.JTextComponent;
import prefuse.activity.Activity;
import prefuse.activity.SlowInSlowOutPacer;
import prefuse.controls.Control;
import prefuse.data.expression.AndPredicate;
import prefuse.data.expression.BooleanLiteral;
import prefuse.data.expression.Predicate;
import prefuse.data.expression.parser.ExpressionParser;
import prefuse.render.Renderer;
import prefuse.util.ColorLib;
import prefuse.util.StringLib;
import prefuse.util.UpdateListener;
import prefuse.util.collections.CopyOnWriteArrayList;
import prefuse.util.display.BackgroundPainter;
import prefuse.util.display.Clip;
import prefuse.util.display.DebugStatsPainter;
import prefuse.util.display.ExportDisplayAction;
import prefuse.util.display.ItemBoundsListener;
import prefuse.util.display.PaintListener;
import prefuse.util.display.RenderingQueue;
import prefuse.visual.VisualItem;
import prefuse.visual.expression.VisiblePredicate;
import prefuse.visual.sort.ItemSorter;

public class Display
  extends JComponent
{
  private static final Logger s_logger = Logger.getLogger(Display.class.getName());
  protected Visualization m_vis;
  protected AndPredicate m_predicate = new AndPredicate();
  protected CopyOnWriteArrayList m_controls = new CopyOnWriteArrayList();
  protected CopyOnWriteArrayList m_painters;
  protected CopyOnWriteArrayList m_bounders;
  protected BufferedImage m_offscreen;
  protected Clip m_clip = new Clip();
  protected Clip m_screen = new Clip();
  protected Clip m_bounds = new Clip();
  protected Rectangle2D m_rclip = new Rectangle2D.Double();
  protected boolean m_damageRedraw = true;
  protected boolean m_highQuality = false;
  protected BackgroundPainter m_bgpainter = null;
  protected RenderingQueue m_queue = new RenderingQueue();
  protected int m_visibleCount = 0;
  protected AffineTransform m_transform = new AffineTransform();
  protected AffineTransform m_itransform = new AffineTransform();
  protected TransformActivity m_transact = new TransformActivity();
  protected Point2D m_tmpPoint = new Point2D.Double();
  protected double frameRate;
  protected int nframes = 0;
  private int sampleInterval = 10;
  private long mark = -1L;
  private JTextComponent m_editor;
  private boolean m_editing;
  private VisualItem m_editItem;
  private String m_editAttribute;
  
  public Display()
  {
    this(null);
  }
  
  public Display(Visualization paramVisualization)
  {
    this(paramVisualization, (Predicate)null);
  }
  
  public Display(Visualization paramVisualization, String paramString)
  {
    this(paramVisualization, (Predicate)ExpressionParser.parse(paramString, true));
  }
  
  public Display(Visualization paramVisualization, Predicate paramPredicate)
  {
    setDoubleBuffered(false);
    setBackground(Color.WHITE);
    this.m_editing = false;
    this.m_editor = new JTextField();
    this.m_editor.setBorder(null);
    this.m_editor.setVisible(false);
    add(this.m_editor);
    InputEventCapturer localInputEventCapturer = new InputEventCapturer();
    addMouseListener(localInputEventCapturer);
    addMouseMotionListener(localInputEventCapturer);
    addMouseWheelListener(localInputEventCapturer);
    addKeyListener(localInputEventCapturer);
    registerDefaultCommands();
    this.m_predicate.addExpressionListener(new UpdateListener()
    {
      protected void update(Object paramAnonymousObject)
      {
        Display.this.damageReport();
      }
    });
    setVisualization(paramVisualization);
    setPredicate(paramPredicate);
    setSize(400, 400);
  }
  
  protected void registerDefaultCommands()
  {
    registerKeyboardAction(new ActionListener()
    {
      private PaintListener m_debug = null;
      
      public void actionPerformed(ActionEvent paramAnonymousActionEvent)
      {
        if (this.m_debug == null)
        {
          this.m_debug = new DebugStatsPainter();
          Display.this.addPaintListener(this.m_debug);
        }
        else
        {
          Display.this.removePaintListener(this.m_debug);
          this.m_debug = null;
        }
        Display.this.repaint();
      }
    }, "debug info", KeyStroke.getKeyStroke("ctrl D"), 0);
    registerKeyboardAction(new ActionListener()
    {
      public void actionPerformed(ActionEvent paramAnonymousActionEvent)
      {
        Display.this.setHighQuality(!Display.this.isHighQuality());
        Display.this.repaint();
      }
    }, "toggle high-quality drawing", KeyStroke.getKeyStroke("ctrl H"), 0);
    try
    {
      registerKeyboardAction(new ExportDisplayAction(this), "export display", KeyStroke.getKeyStroke("ctrl E"), 0);
    }
    catch (SecurityException localSecurityException) {}
  }
  
  public void setSize(int paramInt1, int paramInt2)
  {
    this.m_offscreen = null;
    setPreferredSize(new Dimension(paramInt1, paramInt2));
    super.setSize(paramInt1, paramInt2);
  }
  
  public void setSize(Dimension paramDimension)
  {
    this.m_offscreen = null;
    setPreferredSize(paramDimension);
    super.setSize(paramDimension);
  }
  
  public void invalidate()
  {
    damageReport();
    super.invalidate();
  }
  
  public void setBounds(int paramInt1, int paramInt2, int paramInt3, int paramInt4)
  {
    this.m_offscreen = null;
    super.setBounds(paramInt1, paramInt2, paramInt3, paramInt4);
  }
  
  public void setFont(Font paramFont)
  {
    super.setFont(paramFont);
    this.m_editor.setFont(paramFont);
  }
  
  public double getFrameRate()
  {
    return this.frameRate;
  }
  
  public void setHighQuality(boolean paramBoolean)
  {
    if (this.m_highQuality != paramBoolean) {
      damageReport();
    }
    this.m_highQuality = paramBoolean;
  }
  
  public boolean isHighQuality()
  {
    return this.m_highQuality;
  }
  
  public Visualization getVisualization()
  {
    return this.m_vis;
  }
  
  public void setVisualization(Visualization paramVisualization)
  {
    if (this.m_vis == paramVisualization) {
      return;
    }
    if (this.m_vis != null) {
      this.m_vis.removeDisplay(this);
    }
    this.m_vis = paramVisualization;
    if (this.m_vis != null) {
      this.m_vis.addDisplay(this);
    }
  }
  
  public Predicate getPredicate()
  {
    if (this.m_predicate.size() == 1) {
      return BooleanLiteral.TRUE;
    }
    return this.m_predicate.get(0);
  }
  
  public void setPredicate(String paramString)
  {
    Predicate localPredicate = (Predicate)ExpressionParser.parse(paramString, true);
    setPredicate(localPredicate);
  }
  
  public synchronized void setPredicate(Predicate paramPredicate)
  {
    if (paramPredicate == null) {
      this.m_predicate.set(VisiblePredicate.TRUE);
    } else {
      this.m_predicate.set(new Predicate[] { paramPredicate, VisiblePredicate.TRUE });
    }
  }
  
  public int getVisibleItemCount()
  {
    return this.m_visibleCount;
  }
  
  public ItemSorter getItemSorter()
  {
    return this.m_queue.sort;
  }
  
  public synchronized void setItemSorter(ItemSorter paramItemSorter)
  {
    damageReport();
    this.m_queue.sort = paramItemSorter;
  }
  
  public synchronized void setBackgroundImage(Image paramImage, boolean paramBoolean1, boolean paramBoolean2)
  {
    BackgroundPainter localBackgroundPainter = null;
    if (paramImage != null) {
      localBackgroundPainter = new BackgroundPainter(paramImage, paramBoolean1, paramBoolean2);
    }
    setBackgroundPainter(localBackgroundPainter);
  }
  
  public synchronized void setBackgroundImage(String paramString, boolean paramBoolean1, boolean paramBoolean2)
  {
    BackgroundPainter localBackgroundPainter = null;
    if (paramString != null) {
      localBackgroundPainter = new BackgroundPainter(paramString, paramBoolean1, paramBoolean2);
    }
    setBackgroundPainter(localBackgroundPainter);
  }
  
  private void setBackgroundPainter(BackgroundPainter paramBackgroundPainter)
  {
    if (this.m_bgpainter != null) {
      removePaintListener(this.m_bgpainter);
    }
    this.m_bgpainter = paramBackgroundPainter;
    if (paramBackgroundPainter != null) {
      addPaintListener(paramBackgroundPainter);
    }
  }
  
  public synchronized boolean isDamageRedraw()
  {
    return this.m_damageRedraw;
  }
  
  public synchronized void setDamageRedraw(boolean paramBoolean)
  {
    this.m_damageRedraw = paramBoolean;
    this.m_clip.invalidate();
  }
  
  public synchronized void damageReport(Rectangle2D paramRectangle2D)
  {
    if (this.m_damageRedraw) {
      this.m_clip.union(paramRectangle2D);
    }
  }
  
  public synchronized void damageReport()
  {
    this.m_clip.invalidate();
  }
  
  public synchronized void clearDamage()
  {
    if (this.m_damageRedraw) {
      this.m_clip.reset();
    }
  }
  
  public synchronized Rectangle2D getItemBounds()
  {
    return getItemBounds(new Rectangle2D.Double());
  }
  
  public synchronized Rectangle2D getItemBounds(Rectangle2D paramRectangle2D)
  {
    paramRectangle2D.setFrameFromDiagonal(this.m_bounds.getMinX(), this.m_bounds.getMinY(), this.m_bounds.getMaxX(), this.m_bounds.getMaxY());
    return paramRectangle2D;
  }
  
  public BufferedImage getOffscreenBuffer()
  {
    return this.m_offscreen;
  }
  
  protected BufferedImage getNewOffscreenBuffer(int paramInt1, int paramInt2)
  {
    try
    {
      return (BufferedImage)createImage(paramInt1, paramInt2);
    }
    catch (Exception localException) {}
    return new BufferedImage(paramInt1, paramInt2, 1);
  }
  
  public boolean saveImage(OutputStream paramOutputStream, String paramString, double paramDouble)
  {
    try
    {
      Dimension localDimension = new Dimension((int)(paramDouble * getWidth()), (int)(paramDouble * getHeight()));
      BufferedImage localBufferedImage = getNewOffscreenBuffer(localDimension.width, localDimension.height);
      Graphics2D localGraphics2D = (Graphics2D)localBufferedImage.getGraphics();
      Point2D.Double localDouble = new Point2D.Double(0.0D, 0.0D);
      zoom(localDouble, paramDouble);
      boolean bool = isHighQuality();
      setHighQuality(true);
      paintDisplay(localGraphics2D, localDimension);
      setHighQuality(bool);
      zoom(localDouble, 1.0D / paramDouble);
      ImageIO.write(localBufferedImage, paramString, paramOutputStream);
      return true;
    }
    catch (Exception localException)
    {
      localException.printStackTrace();
    }
    return false;
  }
  
  public void update(Graphics paramGraphics)
  {
    paint(paramGraphics);
  }
  
  protected void paintBufferToScreen(Graphics paramGraphics)
  {
    synchronized (this)
    {
      paramGraphics.drawImage(this.m_offscreen, 0, 0, null);
    }
  }
  
  public void repaintImmediate()
  {
    Graphics localGraphics = getGraphics();
    if ((localGraphics != null) && (this.m_offscreen != null)) {
      paintBufferToScreen(localGraphics);
    }
  }
  
  protected void prepareGraphics(Graphics2D paramGraphics2D)
  {
    if (this.m_transform != null) {
      paramGraphics2D.transform(this.m_transform);
    }
    setRenderingHints(paramGraphics2D);
  }
  
  protected void setRenderingHints(Graphics2D paramGraphics2D)
  {
    if (this.m_highQuality) {
      paramGraphics2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    } else {
      paramGraphics2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
    }
    paramGraphics2D.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
    paramGraphics2D.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
  }
  
  public void paintComponent(Graphics paramGraphics)
  {
    if (this.m_offscreen == null)
    {
      this.m_offscreen = getNewOffscreenBuffer(getWidth(), getHeight());
      damageReport();
    }
    Graphics2D localGraphics2D1 = (Graphics2D)paramGraphics;
    Graphics2D localGraphics2D2 = (Graphics2D)this.m_offscreen.getGraphics();
    paintDisplay(localGraphics2D2, getSize());
    paintBufferToScreen(localGraphics2D1);
    firePostPaint(localGraphics2D1);
    localGraphics2D2.dispose();
    this.nframes += 1;
    if (this.mark < 0L)
    {
      this.mark = System.currentTimeMillis();
      this.nframes = 0;
    }
    else if (this.nframes == this.sampleInterval)
    {
      long l = System.currentTimeMillis();
      this.frameRate = (1000.0D * this.nframes / (l - this.mark));
      this.mark = l;
      this.nframes = 0;
    }
  }
  
  public void paintDisplay(Graphics2D paramGraphics2D, Dimension paramDimension)
  {
    synchronized (this.m_vis)
    {
      synchronized (this)
      {
        if (this.m_clip.isEmpty()) {
          return;
        }
        this.m_screen.setClip(0.0D, 0.0D, paramDimension.width + 1, paramDimension.height + 1);
        this.m_screen.transform(this.m_itransform);
        double d = 1.0D + 1.0D / getScale();
        if (this.m_damageRedraw)
        {
          if (this.m_clip.isInvalid()) {
            this.m_clip.setClip(this.m_screen);
          } else {
            this.m_clip.intersection(this.m_screen);
          }
          this.m_clip.expand(d);
          prepareGraphics(paramGraphics2D);
          this.m_rclip.setFrameFromDiagonal(this.m_clip.getMinX(), this.m_clip.getMinY(), this.m_clip.getMaxX(), this.m_clip.getMaxY());
          paramGraphics2D.setClip(this.m_rclip);
          this.m_rclip.setFrameFromDiagonal(this.m_clip.getMinX() - d, this.m_clip.getMinY() - d, this.m_clip.getMaxX() + d, this.m_clip.getMaxY() + d);
        }
        else
        {
          this.m_rclip.setFrame(0.0D, 0.0D, getWidth(), getHeight());
          this.m_clip.setClip(this.m_screen);
          prepareGraphics(paramGraphics2D);
        }
        clearRegion(paramGraphics2D, this.m_rclip);
        getItemBounds(this.m_rclip);
        this.m_bounds.reset();
        this.m_queue.clear();
        Iterator localIterator = this.m_vis.items(this.m_predicate);
        for (this.m_visibleCount = 0; localIterator.hasNext(); this.m_visibleCount += 1)
        {
          VisualItem localVisualItem = (VisualItem)localIterator.next();
          Rectangle2D localRectangle2D = localVisualItem.getBounds();
          this.m_bounds.union(localRectangle2D);
          if (this.m_clip.intersects(localRectangle2D, d)) {
            this.m_queue.addToRenderQueue(localVisualItem);
          }
          if (localVisualItem.isInteractive()) {
            this.m_queue.addToPickingQueue(localVisualItem);
          }
        }
        this.m_queue.sortRenderQueue();
        for (int i = 0; i < this.m_queue.rsize; i++) {
          this.m_queue.ritems[i].render(paramGraphics2D);
        }
        if (this.m_damageRedraw) {
          this.m_clip.reset();
        }
        checkItemBoundsChanged(this.m_rclip);
      }
    }
  }
  
  public void renderImmediate(VisualItem paramVisualItem)
  {
    Graphics2D localGraphics2D = (Graphics2D)getGraphics();
    prepareGraphics(localGraphics2D);
    paramVisualItem.render(localGraphics2D);
  }
  
  protected void printComponent(Graphics paramGraphics)
  {
    boolean bool = this.m_highQuality;
    try
    {
      this.m_highQuality = true;
      paintDisplay((Graphics2D)paramGraphics, getSize());
    }
    finally
    {
      this.m_highQuality = bool;
    }
  }
  
  protected void clearRegion(Graphics2D paramGraphics2D, Rectangle2D paramRectangle2D)
  {
    paramGraphics2D.setColor(getBackground());
    paramGraphics2D.fill(paramRectangle2D);
    firePrePaint(paramGraphics2D);
  }
  
  public synchronized void setTransform(AffineTransform paramAffineTransform)
    throws NoninvertibleTransformException
  {
    damageReport();
    this.m_transform = paramAffineTransform;
    this.m_itransform = this.m_transform.createInverse();
  }
  
  public AffineTransform getTransform()
  {
    return this.m_transform;
  }
  
  public AffineTransform getInverseTransform()
  {
    return this.m_itransform;
  }
  
  public Point2D getAbsoluteCoordinate(Point2D paramPoint2D1, Point2D paramPoint2D2)
  {
    return this.m_itransform.transform(paramPoint2D1, paramPoint2D2);
  }
  
  public double getScale()
  {
    return this.m_transform.getScaleX();
  }
  
  public double getDisplayX()
  {
    return -this.m_transform.getTranslateX();
  }
  
  public double getDisplayY()
  {
    return -this.m_transform.getTranslateY();
  }
  
  public synchronized void pan(double paramDouble1, double paramDouble2)
  {
    double d1 = paramDouble1 / this.m_transform.getScaleX();
    double d2 = paramDouble2 / this.m_transform.getScaleY();
    panAbs(d1, d2);
  }
  
  public synchronized void panAbs(double paramDouble1, double paramDouble2)
  {
    damageReport();
    this.m_transform.translate(paramDouble1, paramDouble2);
    try
    {
      this.m_itransform = this.m_transform.createInverse();
    }
    catch (Exception localException) {}
  }
  
  public synchronized void panTo(Point2D paramPoint2D)
  {
    this.m_itransform.transform(paramPoint2D, this.m_tmpPoint);
    panToAbs(this.m_tmpPoint);
  }
  
  public synchronized void panToAbs(Point2D paramPoint2D)
  {
    double d1 = this.m_transform.getScaleX();
    double d2 = this.m_transform.getScaleY();
    double d3 = paramPoint2D.getX();
    d3 = Double.isNaN(d3) ? 0.0D : d3;
    double d4 = paramPoint2D.getY();
    d4 = Double.isNaN(d4) ? 0.0D : d4;
    d3 = getWidth() / (2.0D * d1) - d3;
    d4 = getHeight() / (2.0D * d2) - d4;
    double d5 = d3 - this.m_transform.getTranslateX() / d1;
    double d6 = d4 - this.m_transform.getTranslateY() / d2;
    damageReport();
    this.m_transform.translate(d5, d6);
    try
    {
      this.m_itransform = this.m_transform.createInverse();
    }
    catch (Exception localException) {}
  }
  
  public synchronized void zoom(Point2D paramPoint2D, double paramDouble)
  {
    this.m_itransform.transform(paramPoint2D, this.m_tmpPoint);
    zoomAbs(this.m_tmpPoint, paramDouble);
  }
  
  public synchronized void zoomAbs(Point2D paramPoint2D, double paramDouble)
  {
    double d1 = paramPoint2D.getX();
    double d2 = paramPoint2D.getY();
    damageReport();
    this.m_transform.translate(d1, d2);
    this.m_transform.scale(paramDouble, paramDouble);
    this.m_transform.translate(-d1, -d2);
    try
    {
      this.m_itransform = this.m_transform.createInverse();
    }
    catch (Exception localException) {}
  }
  
  public synchronized void rotate(Point2D paramPoint2D, double paramDouble)
  {
    this.m_itransform.transform(paramPoint2D, this.m_tmpPoint);
    rotateAbs(this.m_tmpPoint, paramDouble);
  }
  
  public synchronized void rotateAbs(Point2D paramPoint2D, double paramDouble)
  {
    double d1 = paramPoint2D.getX();
    double d2 = paramPoint2D.getY();
    damageReport();
    this.m_transform.translate(d1, d2);
    this.m_transform.rotate(paramDouble);
    this.m_transform.translate(-d1, -d2);
    try
    {
      this.m_itransform = this.m_transform.createInverse();
    }
    catch (Exception localException) {}
  }
  
  public synchronized void animatePan(double paramDouble1, double paramDouble2, long paramLong)
  {
    double d1 = paramDouble1 / this.m_transform.getScaleX();
    double d2 = paramDouble2 / this.m_transform.getScaleY();
    animatePanAbs(d1, d2, paramLong);
  }
  
  public synchronized void animatePanAbs(double paramDouble1, double paramDouble2, long paramLong)
  {
    this.m_transact.pan(paramDouble1, paramDouble2, paramLong);
  }
  
  public synchronized void animatePanTo(Point2D paramPoint2D, long paramLong)
  {
    Point2D.Double localDouble = new Point2D.Double();
    this.m_itransform.transform(paramPoint2D, localDouble);
    animatePanToAbs(localDouble, paramLong);
  }
  
  public synchronized void animatePanToAbs(Point2D paramPoint2D, long paramLong)
  {
    this.m_tmpPoint.setLocation(0.0D, 0.0D);
    this.m_itransform.transform(this.m_tmpPoint, this.m_tmpPoint);
    double d1 = paramPoint2D.getX();
    d1 = Double.isNaN(d1) ? 0.0D : d1;
    double d2 = paramPoint2D.getY();
    d2 = Double.isNaN(d2) ? 0.0D : d2;
    double d3 = getWidth() / (2.0D * this.m_transform.getScaleX());
    double d4 = getHeight() / (2.0D * this.m_transform.getScaleY());
    double d5 = d3 - d1 + this.m_tmpPoint.getX();
    double d6 = d4 - d2 + this.m_tmpPoint.getY();
    animatePanAbs(d5, d6, paramLong);
  }
  
  public synchronized void animateZoom(Point2D paramPoint2D, double paramDouble, long paramLong)
  {
    Point2D.Double localDouble = new Point2D.Double();
    this.m_itransform.transform(paramPoint2D, localDouble);
    animateZoomAbs(localDouble, paramDouble, paramLong);
  }
  
  public synchronized void animateZoomAbs(Point2D paramPoint2D, double paramDouble, long paramLong)
  {
    this.m_transact.zoom(paramPoint2D, paramDouble, paramLong);
  }
  
  public synchronized void animatePanAndZoomTo(Point2D paramPoint2D, double paramDouble, long paramLong)
  {
    Point2D.Double localDouble = new Point2D.Double();
    this.m_itransform.transform(paramPoint2D, localDouble);
    animatePanAndZoomToAbs(localDouble, paramDouble, paramLong);
  }
  
  public synchronized void animatePanAndZoomToAbs(Point2D paramPoint2D, double paramDouble, long paramLong)
  {
    this.m_transact.panAndZoom(paramPoint2D, paramDouble, paramLong);
  }
  
  public boolean isTranformInProgress()
  {
    return this.m_transact.isRunning();
  }
  
  public void addPaintListener(PaintListener paramPaintListener)
  {
    if (this.m_painters == null) {
      this.m_painters = new CopyOnWriteArrayList();
    }
    this.m_painters.add(paramPaintListener);
  }
  
  public void removePaintListener(PaintListener paramPaintListener)
  {
    this.m_painters.remove(paramPaintListener);
  }
  
  protected void firePrePaint(Graphics2D paramGraphics2D)
  {
    if ((this.m_painters != null) && (this.m_painters.size() > 0))
    {
      Object[] arrayOfObject = this.m_painters.getArray();
      for (int i = 0; i < arrayOfObject.length; i++) {
        try
        {
          ((PaintListener)arrayOfObject[i]).prePaint(this, paramGraphics2D);
        }
        catch (Exception localException)
        {
          s_logger.warning("Exception thrown by PaintListener: " + localException + "\n" + StringLib.getStackTrace(localException));
        }
      }
    }
  }
  
  protected void firePostPaint(Graphics2D paramGraphics2D)
  {
    if ((this.m_painters != null) && (this.m_painters.size() > 0))
    {
      Object[] arrayOfObject = this.m_painters.getArray();
      for (int i = 0; i < arrayOfObject.length; i++) {
        try
        {
          ((PaintListener)arrayOfObject[i]).postPaint(this, paramGraphics2D);
        }
        catch (Exception localException)
        {
          s_logger.warning("Exception thrown by PaintListener: " + localException + "\n" + StringLib.getStackTrace(localException));
        }
      }
    }
  }
  
  public void addItemBoundsListener(ItemBoundsListener paramItemBoundsListener)
  {
    if (this.m_bounders == null) {
      this.m_bounders = new CopyOnWriteArrayList();
    }
    this.m_bounders.add(paramItemBoundsListener);
  }
  
  public void removeItemBoundsListener(ItemBoundsListener paramItemBoundsListener)
  {
    this.m_bounders.remove(paramItemBoundsListener);
  }
  
  protected void checkItemBoundsChanged(Rectangle2D paramRectangle2D)
  {
    if (this.m_bounds.equals(paramRectangle2D)) {
      return;
    }
    if ((this.m_bounders != null) && (this.m_bounders.size() > 0))
    {
      Object[] arrayOfObject = this.m_bounders.getArray();
      for (int i = 0; i < arrayOfObject.length; i++) {
        try
        {
          ((ItemBoundsListener)arrayOfObject[i]).itemBoundsChanged(this);
        }
        catch (Exception localException)
        {
          s_logger.warning("Exception thrown by ItemBoundsListener: " + localException + "\n" + StringLib.getStackTrace(localException));
        }
      }
    }
  }
  
  public void addControlListener(Control paramControl)
  {
    this.m_controls.add(paramControl);
  }
  
  public void removeControlListener(Control paramControl)
  {
    this.m_controls.remove(paramControl);
  }
  
  public synchronized VisualItem findItem(Point paramPoint)
  {
    Point2D localPoint2D = this.m_itransform == null ? paramPoint : this.m_itransform.transform(paramPoint, this.m_tmpPoint);
    if (!this.m_queue.psorted) {
      this.m_queue.sortPickingQueue();
    }
    int i = this.m_queue.psize;
    for (;;)
    {
      i--;
      if (i < 0) {
        break;
      }
      VisualItem localVisualItem = this.m_queue.pitems[i];
      if (localVisualItem.isValid())
      {
        Renderer localRenderer = localVisualItem.getRenderer();
        if ((localRenderer != null) && (localVisualItem.isInteractive()) && (localRenderer.locatePoint(localPoint2D, localVisualItem))) {
          return localVisualItem;
        }
      }
    }
    return null;
  }
  
  public JTextComponent getTextEditor()
  {
    return this.m_editor;
  }
  
  public void setTextEditor(JTextComponent paramJTextComponent)
  {
    remove(this.m_editor);
    this.m_editor = paramJTextComponent;
    add(this.m_editor, 1);
  }
  
  public void editText(VisualItem paramVisualItem, String paramString)
  {
    if (this.m_editing) {
      stopEditing();
    }
    Rectangle2D localRectangle2D = paramVisualItem.getBounds();
    Rectangle localRectangle = this.m_transform.createTransformedShape(localRectangle2D).getBounds();
    if ((this.m_editor instanceof JTextArea))
    {
      localRectangle.y -= 2;
      localRectangle.width += 22;
      localRectangle.height += 2;
    }
    else
    {
      localRectangle.x += 3;
      localRectangle.y += 1;
      localRectangle.width -= 5;
      localRectangle.height -= 2;
    }
    Font localFont1 = getFont();
    int i = (int)Math.round(localFont1.getSize() * this.m_transform.getScaleX());
    Font localFont2 = new Font(localFont1.getFontName(), localFont1.getStyle(), i);
    this.m_editor.setFont(localFont2);
    editText(paramVisualItem, paramString, localRectangle);
  }
  
  public void editText(VisualItem paramVisualItem, String paramString, Rectangle paramRectangle)
  {
    if (this.m_editing) {
      stopEditing();
    }
    String str = paramVisualItem.getString(paramString);
    this.m_editItem = paramVisualItem;
    this.m_editAttribute = paramString;
    Color localColor1 = ColorLib.getColor(paramVisualItem.getTextColor());
    Color localColor2 = ColorLib.getColor(paramVisualItem.getFillColor());
    this.m_editor.setForeground(localColor1);
    this.m_editor.setBackground(localColor2);
    editText(str, paramRectangle);
  }
  
  public void editText(String paramString, Rectangle paramRectangle)
  {
    if (this.m_editing) {
      stopEditing();
    }
    this.m_editing = true;
    this.m_editor.setBounds(paramRectangle.x, paramRectangle.y, paramRectangle.width, paramRectangle.height);
    this.m_editor.setText(paramString);
    this.m_editor.setVisible(true);
    this.m_editor.setCaretPosition(paramString.length());
    this.m_editor.requestFocus();
  }
  
  public void stopEditing()
  {
    this.m_editor.setVisible(false);
    if (this.m_editItem != null)
    {
      String str = this.m_editor.getText();
      this.m_editItem.set(str, this.m_editAttribute);
      this.m_editItem = null;
      this.m_editAttribute = null;
      this.m_editor.setBackground(null);
      this.m_editor.setForeground(null);
    }
    this.m_editing = false;
  }
  
  public class InputEventCapturer
    implements MouseMotionListener, MouseWheelListener, MouseListener, KeyListener
  {
    private VisualItem activeItem = null;
    private boolean mouseDown = false;
    
    public InputEventCapturer() {}
    
    private boolean validityCheck()
    {
      if (this.activeItem.isValid()) {
        return true;
      }
      this.activeItem = null;
      return false;
    }
    
    public void mouseDragged(MouseEvent paramMouseEvent)
    {
      synchronized (Display.this.m_vis)
      {
        if (this.activeItem != null)
        {
          if (validityCheck()) {
            fireItemDragged(this.activeItem, paramMouseEvent);
          }
        }
        else {
          fireMouseDragged(paramMouseEvent);
        }
      }
    }
    
    public void mouseMoved(MouseEvent paramMouseEvent)
    {
      synchronized (Display.this.m_vis)
      {
        int i = 0;
        VisualItem localVisualItem = Display.this.findItem(paramMouseEvent.getPoint());
        if ((this.activeItem != null) && (this.activeItem != localVisualItem))
        {
          if (validityCheck()) {
            fireItemExited(this.activeItem, paramMouseEvent);
          }
          i = 1;
        }
        if ((localVisualItem != null) && (localVisualItem != this.activeItem))
        {
          fireItemEntered(localVisualItem, paramMouseEvent);
          i = 1;
        }
        this.activeItem = localVisualItem;
        if (i != 0) {
          return;
        }
        if ((localVisualItem != null) && (localVisualItem == this.activeItem)) {
          fireItemMoved(localVisualItem, paramMouseEvent);
        }
        if (localVisualItem == null) {
          fireMouseMoved(paramMouseEvent);
        }
      }
    }
    
    public void mouseWheelMoved(MouseWheelEvent paramMouseWheelEvent)
    {
      synchronized (Display.this.m_vis)
      {
        if (this.activeItem != null)
        {
          if (validityCheck()) {
            fireItemWheelMoved(this.activeItem, paramMouseWheelEvent);
          }
        }
        else {
          fireMouseWheelMoved(paramMouseWheelEvent);
        }
      }
    }
    
    public void mouseClicked(MouseEvent paramMouseEvent)
    {
      synchronized (Display.this.m_vis)
      {
        if (this.activeItem != null)
        {
          if (validityCheck()) {
            fireItemClicked(this.activeItem, paramMouseEvent);
          }
        }
        else {
          fireMouseClicked(paramMouseEvent);
        }
      }
    }
    
    public void mousePressed(MouseEvent paramMouseEvent)
    {
      synchronized (Display.this.m_vis)
      {
        this.mouseDown = true;
        if (this.activeItem != null)
        {
          if (validityCheck()) {
            fireItemPressed(this.activeItem, paramMouseEvent);
          }
        }
        else {
          fireMousePressed(paramMouseEvent);
        }
      }
    }
    
    public void mouseReleased(MouseEvent paramMouseEvent)
    {
      synchronized (Display.this.m_vis)
      {
        if (this.activeItem != null)
        {
          if (validityCheck()) {
            fireItemReleased(this.activeItem, paramMouseEvent);
          }
        }
        else {
          fireMouseReleased(paramMouseEvent);
        }
        if ((this.activeItem != null) && (this.mouseDown) && (isOffComponent(paramMouseEvent)))
        {
          fireItemExited(this.activeItem, paramMouseEvent);
          this.activeItem = null;
        }
        this.mouseDown = false;
      }
    }
    
    public void mouseEntered(MouseEvent paramMouseEvent)
    {
      synchronized (Display.this.m_vis)
      {
        fireMouseEntered(paramMouseEvent);
      }
    }
    
    public void mouseExited(MouseEvent paramMouseEvent)
    {
      synchronized (Display.this.m_vis)
      {
        if ((!this.mouseDown) && (this.activeItem != null))
        {
          fireItemExited(this.activeItem, paramMouseEvent);
          this.activeItem = null;
        }
        fireMouseExited(paramMouseEvent);
      }
    }
    
    public void keyPressed(KeyEvent paramKeyEvent)
    {
      synchronized (Display.this.m_vis)
      {
        if (this.activeItem != null)
        {
          if (validityCheck()) {
            fireItemKeyPressed(this.activeItem, paramKeyEvent);
          }
        }
        else {
          fireKeyPressed(paramKeyEvent);
        }
      }
    }
    
    public void keyReleased(KeyEvent paramKeyEvent)
    {
      synchronized (Display.this.m_vis)
      {
        if (this.activeItem != null)
        {
          if (validityCheck()) {
            fireItemKeyReleased(this.activeItem, paramKeyEvent);
          }
        }
        else {
          fireKeyReleased(paramKeyEvent);
        }
      }
    }
    
    public void keyTyped(KeyEvent paramKeyEvent)
    {
      synchronized (Display.this.m_vis)
      {
        if (this.activeItem != null)
        {
          if (validityCheck()) {
            fireItemKeyTyped(this.activeItem, paramKeyEvent);
          }
        }
        else {
          fireKeyTyped(paramKeyEvent);
        }
      }
    }
    
    private boolean isOffComponent(MouseEvent paramMouseEvent)
    {
      int i = paramMouseEvent.getX();
      int j = paramMouseEvent.getY();
      return (i < 0) || (i > Display.this.getWidth()) || (j < 0) || (j > Display.this.getHeight());
    }
    
    private void fireItemDragged(VisualItem paramVisualItem, MouseEvent paramMouseEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.itemDragged(paramVisualItem, paramMouseEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireItemMoved(VisualItem paramVisualItem, MouseEvent paramMouseEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.itemMoved(paramVisualItem, paramMouseEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireItemWheelMoved(VisualItem paramVisualItem, MouseWheelEvent paramMouseWheelEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.itemWheelMoved(paramVisualItem, paramMouseWheelEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireItemClicked(VisualItem paramVisualItem, MouseEvent paramMouseEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.itemClicked(paramVisualItem, paramMouseEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireItemPressed(VisualItem paramVisualItem, MouseEvent paramMouseEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.itemPressed(paramVisualItem, paramMouseEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireItemReleased(VisualItem paramVisualItem, MouseEvent paramMouseEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.itemReleased(paramVisualItem, paramMouseEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireItemEntered(VisualItem paramVisualItem, MouseEvent paramMouseEvent)
    {
      paramVisualItem.setHover(true);
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.itemEntered(paramVisualItem, paramMouseEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireItemExited(VisualItem paramVisualItem, MouseEvent paramMouseEvent)
    {
      paramVisualItem.setHover(false);
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.itemExited(paramVisualItem, paramMouseEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireItemKeyPressed(VisualItem paramVisualItem, KeyEvent paramKeyEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      if (arrayOfObject.length == 0) {
        return;
      }
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.itemKeyPressed(paramVisualItem, paramKeyEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireItemKeyReleased(VisualItem paramVisualItem, KeyEvent paramKeyEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.itemKeyReleased(paramVisualItem, paramKeyEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireItemKeyTyped(VisualItem paramVisualItem, KeyEvent paramKeyEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.itemKeyTyped(paramVisualItem, paramKeyEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireMouseEntered(MouseEvent paramMouseEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.mouseEntered(paramMouseEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireMouseExited(MouseEvent paramMouseEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.mouseExited(paramMouseEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireMousePressed(MouseEvent paramMouseEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.mousePressed(paramMouseEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireMouseReleased(MouseEvent paramMouseEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.mouseReleased(paramMouseEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireMouseClicked(MouseEvent paramMouseEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.mouseClicked(paramMouseEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireMouseDragged(MouseEvent paramMouseEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.mouseDragged(paramMouseEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireMouseMoved(MouseEvent paramMouseEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.mouseMoved(paramMouseEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireMouseWheelMoved(MouseWheelEvent paramMouseWheelEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.mouseWheelMoved(paramMouseWheelEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireKeyPressed(KeyEvent paramKeyEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.keyPressed(paramKeyEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireKeyReleased(KeyEvent paramKeyEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.keyReleased(paramKeyEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
    
    private void fireKeyTyped(KeyEvent paramKeyEvent)
    {
      Object[] arrayOfObject = Display.this.m_controls.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        Control localControl = (Control)arrayOfObject[i];
        if (localControl.isEnabled()) {
          try
          {
            localControl.keyTyped(paramKeyEvent);
          }
          catch (Exception localException)
          {
            Display.s_logger.warning("Exception thrown by Control: " + localException + "\n" + StringLib.getStackTrace(localException));
          }
        }
      }
    }
  }
  
  private class TransformActivity
    extends Activity
  {
    private double[] src = new double[6];
    private double[] dst = new double[6];
    private AffineTransform m_at = new AffineTransform();
    
    public TransformActivity()
    {
      super(20L, 0L);
      setPacingFunction(new SlowInSlowOutPacer());
    }
    
    private AffineTransform getTransform()
    {
      if (isScheduled()) {
        this.m_at.setTransform(this.dst[0], this.dst[1], this.dst[2], this.dst[3], this.dst[4], this.dst[5]);
      } else {
        this.m_at.setTransform(Display.this.m_transform);
      }
      return this.m_at;
    }
    
    public void panAndZoom(Point2D paramPoint2D, double paramDouble, long paramLong)
    {
      AffineTransform localAffineTransform = getTransform();
      cancel();
      setDuration(paramLong);
      Display.this.m_tmpPoint.setLocation(0.0D, 0.0D);
      Display.this.m_itransform.transform(Display.this.m_tmpPoint, Display.this.m_tmpPoint);
      double d1 = paramPoint2D.getX();
      d1 = Double.isNaN(d1) ? 0.0D : d1;
      double d2 = paramPoint2D.getY();
      d2 = Double.isNaN(d2) ? 0.0D : d2;
      double d3 = Display.this.getWidth() / (2.0D * Display.this.m_transform.getScaleX());
      double d4 = Display.this.getHeight() / (2.0D * Display.this.m_transform.getScaleY());
      double d5 = d3 - d1 + Display.this.m_tmpPoint.getX();
      double d6 = d4 - d2 + Display.this.m_tmpPoint.getY();
      localAffineTransform.translate(d5, d6);
      localAffineTransform.translate(paramPoint2D.getX(), paramPoint2D.getY());
      localAffineTransform.scale(paramDouble, paramDouble);
      localAffineTransform.translate(-paramPoint2D.getX(), -paramPoint2D.getY());
      localAffineTransform.getMatrix(this.dst);
      Display.this.m_transform.getMatrix(this.src);
      run();
    }
    
    public void pan(double paramDouble1, double paramDouble2, long paramLong)
    {
      AffineTransform localAffineTransform = getTransform();
      cancel();
      setDuration(paramLong);
      localAffineTransform.translate(paramDouble1, paramDouble2);
      localAffineTransform.getMatrix(this.dst);
      Display.this.m_transform.getMatrix(this.src);
      run();
    }
    
    public void zoom(Point2D paramPoint2D, double paramDouble, long paramLong)
    {
      AffineTransform localAffineTransform = getTransform();
      cancel();
      setDuration(paramLong);
      double d1 = paramPoint2D.getX();
      double d2 = paramPoint2D.getY();
      localAffineTransform.translate(d1, d2);
      localAffineTransform.scale(paramDouble, paramDouble);
      localAffineTransform.translate(-d1, -d2);
      localAffineTransform.getMatrix(this.dst);
      Display.this.m_transform.getMatrix(this.src);
      run();
    }
    
    protected void run(long paramLong)
    {
      double d = getPace(paramLong);
      Display.this.damageReport();
      Display.this.m_transform.setTransform(this.src[0] + d * (this.dst[0] - this.src[0]), this.src[1] + d * (this.dst[1] - this.src[1]), this.src[2] + d * (this.dst[2] - this.src[2]), this.src[3] + d * (this.dst[3] - this.src[3]), this.src[4] + d * (this.dst[4] - this.src[4]), this.src[5] + d * (this.dst[5] - this.src[5]));
      try
      {
        Display.this.m_itransform = Display.this.m_transform.createInverse();
      }
      catch (Exception localException) {}
      Display.this.repaint();
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\Display.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */